# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

project = "AutoRAG"
copyright = "2024, Marker-Inc"
author = "Marker-Inc"

with open("../../autorag/VERSION") as f:
	version = f.read().strip()

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = [
	"sphinx.ext.autodoc",
	"sphinx.ext.extlinks",
	"sphinx.ext.intersphinx",
	"sphinx.ext.mathjax",
	"sphinx.ext.todo",
	"sphinx.ext.viewcode",
	"myst_parser",
	"sphinx_copybutton",
	"sphinx_design",
	"sphinx_inline_tabs",
	"sphinxcontrib.googleanalytics",
	"sphinx_sitemap",
]
source_suffix = {
	".rst": "restructuredtext",
	".md": "markdown",
}

templates_path = ["_templates"]
exclude_patterns = []

googleanalytics_id = "G-T8ZKQM6RT9"

# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = "furo"
html_static_path = ["_static"]
html_baseurl = "https://docs.auto-rag.com/"
